#!/bin/bash

tot=0
passed=0

MD5SUM=md5sum

echo "1" | $MD5SUM >& /dev/null
if [ "$?" -ne "0" ]; then
	echo "ERROR: cannot perform tests since the  $MD5SUM program cannot be run"
	exit
fi
if [ ! -e ./lencod.exe ]; then
	echo "ERROR: Cannot perform tests if the software is not compiled. Please build  lencod.exe in the current directory"
	exit
fi
if [ ! -e ./ldecod.exe ]; then
	echo "ERROR: Cannot perform tests if the software is not compiled. Please build  ldecod.exe in the current directory"
	exit
fi

check_test() {
	tot=$[$tot+1]
	m=`$MD5SUM "$1" | cut -d" " -f 1`
	if [ "$m" != "$2" ]; then
		echo 
		echo "#######################################"
		echo "TEST: $3 FAILED!"
		echo "#######################################"
		exit
	else
		echo "TEST: $3 PASSED."
		passed=$[$passed+1]
	fi
}

f1=seq_EM_JEG_test1.rtp
rm -f "$f1"
./lencod.exe -d EM_JEG_encoder_test1.cfg
check_test $f1 05548e367ebe4a5b111904528639c63c "Encoding of bitstream"

f3=seq_EM_JEG_test1_concealed.yuv
rm -f "$f3"
./ldecod.exe -d EM_JEG_decoder_test1.cfg -loss EM_JEG_noloss.txt
check_test $f3 e8d77ace2a28a0317a471374331abc30 ""

rm -f "$f3"
./ldecod.exe -d EM_JEG_decoder_test1.cfg -loss EM_JEG_lossP.txt
check_test $f3 6ef03bbfb1bcd60987d1d9dedcccd888 ""

rm -f "$f3"
./ldecod.exe -d EM_JEG_decoder_test1.cfg -loss EM_JEG_lossB.txt
check_test $f3 a465804cb9946b582ebca7fef84415f9 ""

f4=seq_EM_JEG_test1_rec.yuv

echo
echo "TEST: "$passed"/"$tot" tests PASSED."
echo

echo "Cleaned up"
rm -f "$f1" "$f3" "$f4"
rm -f seq_EM_JEG_test1.rtp seq_EM_JEG_test1_rec.yuv seq_EM_JEG_test1_concealed.yuv 


