/*
 * Decoder modifications to consider data loss while decoding frames. Source code compatible with JM 16.1
 * Author: Enrico Masala <masala@polito.it>, Politecnico di Torino, Jan 3, 2014
 *
 * Disclaimer:
 * This software is meant to be a prototype only, it is not optimized or fitted to any purposes.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 *
 * Citing this work:
 * We kindly ask people using this code in their research activity to cite a work
 * in which the approach employed by this software is used, e.g.,
 * E. Masala, A. Vesco, M. Baldi, J.C. De Martin,
 * "Optimized H.264 Video Encoding and Packetization for Video Transmission Over Pipeline Forwarding Networks",
 * IEEE Transactions on Multimedia, vol. 11, n. 5, Aug 2009, pp. 972-985.
 * 
 * This work has also been contributed to the VQEG JEG-Hybrid group, see also the meeting input document at:
 * ftp://vqeg.its.bldrdoc.gov/Documents/VQEG_Boulder_Jan14/MeetingFiles/
 * VQEG_JEG-Hybrid_2014__005_Politecnico_di_Torino_contribution_Simulation_of_Robust_H.264_AVC_Decoding.pdf
 * 
 * For more up-to-date information and publications please visit: http://media.polito.it/jeg
 *
 * $Id$
 *
 */

#ifndef __EM_JEG_modifications_h_
#define __EM_JEG_modifications_h_

#include <stdio.h>

#include "global.h"
#include "mbuffer.h"

#define LOSSTYPE_AFTER_SLICE 1
#define LOSSTYPE_AFTER_FRAME 2
#define LOSSTYPE_AFTER_MB_DELAY 3

typedef struct EM_JEG_fd_s {
	FILE *loss;
} EM_JEG_fd_t;

typedef struct EM_JEG_mb_info_s {
	int mb_addr;
	int type;
} EM_JEG_mb_info_t;

typedef struct EM_JEG_slice_info_s {
	EM_JEG_mb_info_t *mb;  // pointer to an array of MB info
	int num_mb;
} EM_JEG_slice_info_t;

typedef struct EM_JEG_dpb_info_s {
	int poc;
	int valid;
	int type;
} EM_JEG_dpb_info_t;

// These functions are called in the main
void EM_JEG_init(InputParameters *p_Inp);
void EM_JEG_free(void);

FILE *EM_JEG_open_file_slicesjeg(char *fname);
FILE *EM_JEG_open_file_loss(char *fname);
void EM_JEG_close_file(FILE *fp);


void EM_JEG_add_new_mb(int mb_addr);
void EM_JEG_apply_slice_concealment(Slice *currSlice);
void EM_JEG_apply_mb_concealment(VideoParameters *img, imgpel **imgY, imgpel ***imgUV, StorablePicture *refPic, int mb_address);
int EM_JEG_isMBConcealed(int idx);

void EM_JEG_compute_mb_coordinates(VideoParameters *img, int *mb_x, int *mb_y, int mb_address);
char *EM_JEG_toString_imgtype(int type);
EM_JEG_slice_info_t *EM_JEG_get_slice_info(void);
int EM_JEG_get_loss_event(void);

void EM_JEG_init_buffers(void);
void EM_JEG_init_frame(StorablePicture *dec_picture);
void EM_JEG_save_frame(StorablePicture *dec_picture);

void EM_JEG_dpb_print(void);
void EM_JEG_dpb_remove_oldest_picture(void);
int EM_JEG_dpb_find_free_element(void);
int EM_JEG_dpb_find_maxframe_and_type(int fr, int isIorP);

#endif
  
